@@ -20,7 +20,8 @@ Search through the project for method declarations, invocants or free
 text using Ack.
 
 Run tests and scripts with easy navigation to errors/warnings/failing
-tests.
+tests. Tests can be run under Devel::Cover to collect (and display)
+test coverage information.
 
 Automate common editing tasks related to source code, tests, regular
 expressions, etc.
@@ -108,6 +109,21 @@ Compilation mode and the settings appropriate for the source type
 (Test, Module, etc.). Highlight errors and jump to source with C-c
 C-c.
 
+B<Run file under Devel::CoverX::Covered> -- C<C-o r c> -- Run the
+current file, collecting Devel::CoverX::Covered information.
+
+B<Edit - Copy Package Name> -- C<C-o e c p> -- Copy the current package name.
+
+B<Edit - Copy Package Name From File> -- C<C-o e c P> -- Copy the
+current package name from file name.
+
+B<Edit - Copy Sub Name> -- C<C-o e c s> -- Copy the current sub name.
+
+B<Edit - Copy Method Name> -- C<C-o e c m> -- Copy the current method
+name (i.e. package->sub).
+
+B<Edit - Copy File Name> -- C<C-o e c f> -- Copy the current file name.
+
 B<Edit - Add Use Statement> -- C<C-o e a u> -- Add a 'use Module'
 statement to the 'use Module' section at the top. Default Module name
 is module at point.
@@ -115,6 +131,18 @@ is module at point.
 B<Edit - Move Use Statement> -- C<C-o e m u> -- Move the 'use Module'
 statement at point to the 'use Module' section at the top.
 
+B<Extract Variable> - C<C-o e e v> -- Do the refactoring Extract
+Variable of the active region.
+
+B<Find Callers> - C<C-o e f c> -- Find callers of a method in the
+project and insert the call tree as a comment in the source.
+
+B<Visualize Callers> - C<C-o e v c> -- Visualize the callers comment
+created by "Find Callers" using GrapViz.
+
+B<Visualize Callers> - C<C-o e v c> -- Visualize callers in a call
+tree (found by Find Callers) by drawing the call tree using GraphViz.
+
 B<Edit Test Count> -- C<C-o e t c> -- Increase the test count
 (e.g. "tests => 43")
 
@@ -233,7 +261,8 @@ These aren't needed to begin with, but may be very useful.
 =item * L<Devel::CoverX::Covered>
 
 If you have a lot of tests to navigate and run a nightly build with
-Devel::Cover to generate test coverage.
+Devel::Cover to generate test coverage. You can also run individual
+files under Devel::CoverX::Covered with C<C-o r c>.
 
 =item * L<File::Corresponding>
 
@@ -370,7 +399,7 @@ These are the ones I use every day so they may be a good start:
 
 =item * Find with Ack
 
-=item * Find sub declerations
+=item * Find sub declarations
 
 =back
 
@@ -388,7 +417,7 @@ These are the ones I use every day so they may be a good start:
 =head3 Smart docs
 
 
-=for html <p>[ <a href="http://search.cpan.org/src/JOHANL/Devel-PerlySense-0.0183/doc/smart_docs_method.html">Screenshot</a> ]<p>
+=for html <p>[ <a href="http://search.cpan.org/src/JOHANL/Devel-PerlySense-0.0217/doc/smart_docs_method.html">Screenshot</a> ]<p>
 
 
 C<C-o C-d> is the "Smart docs" command. It brings up POD documentation
@@ -534,8 +563,8 @@ See also:
 
 =item * Git -- Magit
 
-This requires you to have Magit installed. Download and manual at:
-L<http://zagadka.vm.bytemark.co.uk/magit/>.
+This requires you to have Magit installed. Install using ELPA (C<M-x
+packages-list-packages>). Docs at L<https://magit.vc/>.
 
 When you switch to an existing Magit status buffer the status is
 refreshed automatically to display the current status.
@@ -570,7 +599,7 @@ See L<File::Corresponding> for details.
 =head3 Find with Ack
 
 
-=for html <p>[ <a href="http://search.cpan.org/src/JOHANL/Devel-PerlySense-0.0183/doc/find_with_ack.html">Screenshot</a> ]<p>
+=for html <p>[ <a href="http://search.cpan.org/src/JOHANL/Devel-PerlySense-0.0217/doc/find_with_ack.html">Screenshot</a> ]<p>
 
 
 C<C-o f a> -- Ack through the source and display the hits in a
@@ -590,14 +619,18 @@ you'll get to edit the command line with a sensible default chosen from:
 
 When editing the ack command you can use the following keys to set options
 
- |---------+--------+--------+------------------------------------------|
- | "C-o w" | toggle | -w     | Whole word                               |
- | "C-o q" | toggle | -Q     | Quote metacharacters, pattern is literal |
- | "C-o i" | toggle | -i     | Ignore case                              |
- | "C-o a" | use    | --all  |                                          |
- | "C-o p" | use    | --perl |                                          |
- | "C-o s" | use    | --sql  |                                          |
- |---------+--------+--------+------------------------------------------|
+ |---------+--------+---------------+------------------------------------------|
+ | "C-o w" | toggle | -w            | Whole word                               |
+ | "C-o q" | toggle | -Q            | Quote metacharacters, pattern is literal |
+ | "C-o i" | toggle | -i            | Ignore case                              |
+ | "C-o p" | use    | --perl        |                                          |
+ | "C-o a" | use    | --all         | Ack version <  2.0                       |
+ | "C-o k" | use    | --known-types |                                          |
+ | "C-o s" | use    | --sql         |                                          |
+ |---------+--------+---------------+------------------------------------------|
+
+To search for all files, toggle the current file type off (typically
+--perl, so type C<C-o p> to toggle it off).
 
 For details, refer to the L<ack> documentation (the program was
 installed as a dependency of PerlySense).
@@ -616,7 +649,7 @@ something else using C<M-x rename-buffer>.
 =head3 Find sub declarations
 
 
-=for html <p>[ <a href="http://search.cpan.org/src/JOHANL/Devel-PerlySense-0.0183/doc/find_sub_declaration.html">Screenshot</a> ]<p>
+=for html <p>[ <a href="http://search.cpan.org/src/JOHANL/Devel-PerlySense-0.0217/doc/find_sub_declaration.html">Screenshot</a> ]<p>
 
 
 C<C-o f s> -- Ack the Project for I<sub declarations> of the method,
@@ -631,7 +664,7 @@ the object (C<$us|er_agent-E<gt>get()>).
 =head3 Find method calls
 
 
-=for html <p>[ <a href="http://search.cpan.org/src/JOHANL/Devel-PerlySense-0.0183/doc/find_method_calls.html">Screenshot</a> ]<p>
+=for html <p>[ <a href="http://search.cpan.org/src/JOHANL/Devel-PerlySense-0.0217/doc/find_method_calls.html">Screenshot</a> ]<p>
 
 
 C<C-o f c> -- Ack the Project for I<method calls> to the method, or
@@ -774,7 +807,7 @@ q -- Quit the Class Overview buffer.
 =head2 Testing
 
 
-=for html <p>[ <a href="http://search.cpan.org/src/JOHANL/Devel-PerlySense-0.0183/doc/testing.html">Screenshot</a> ]<p>
+=for html <p>[ <a href="http://search.cpan.org/src/JOHANL/Devel-PerlySense-0.0217/doc/testing.html">Screenshot</a> ]<p>
 
 
 =head3 Run File
@@ -832,6 +865,15 @@ Note: this will re-run whatever is displayed in the B<*compilation*>
 buffer.
 
 
+=head3 Run File under Devel::CoverX::Covered
+
+C<C-o r c> -- This is the same as Run File, but collect test coverage
+information using Devel::CoverX::Covered.
+
+Note: Currently this only works with Unix like shells.
+
+
+
 =head3 Go to Run-buffer
 
 Invoke C<C-o g r> to go to the B<*compilation*> buffer.
@@ -882,7 +924,7 @@ it.
 =head3 Go to Tests - Other Files
 
 
-=for html <p>[ <a href="http://search.cpan.org/src/JOHANL/Devel-PerlySense-0.0183/doc/goto_tests.html">Screenshot</a> ]<p>
+=for html <p>[ <a href="http://search.cpan.org/src/JOHANL/Devel-PerlySense-0.0217/doc/goto_tests.html">Screenshot</a> ]<p>
 
 
 C<C-o g t o> -- In a test file, navigate to the source files that are
@@ -895,6 +937,9 @@ limited to those that cover that particular sub.
 This requires that L<Devel::CoverX::Covered> is installed and a
 L<Devel::Cover> cover_db in the project root directory.
 
+You can build the coverage database either as a (very slow) separate
+test run, or by running individual files with C<C-o r c>.
+
 See L<Devel::CoverX::Covered> for details.
 
 
@@ -993,7 +1038,7 @@ first).
 =head3 Flymake Introduction
 
 
-=for html <p>[ <a href="http://search.cpan.org/src/JOHANL/Devel-PerlySense-0.0183/doc/flymake.html">Screenshot</a> ]<p>
+=for html <p>[ <a href="http://search.cpan.org/src/JOHANL/Devel-PerlySense-0.0217/doc/flymake.html">Screenshot</a> ]<p>
 
 
 "Flymake performs on-the-fly syntax checks of the files being edited
@@ -1048,12 +1093,12 @@ activities.
 Set "syntax" and/or "critic" to 1 to enable them.
 
 B<The primary reason "syntax" is turned off by default is that it's a
-potential security hole>; running "perl -c" on a file will not only
-check the syntax; BEGIN and CHECK blocks are also executed. Doing that
-on random code may be considered... baaad.
+potential security hole>; running C<perl -c> on a file will not only
+check the syntax; C<BEGIN> and C<CHECK> blocks are also
+executed. Doing that on random code may be considered... baaad.
 
-This way you can have Flymake enabled globally and still not run "perl
--c" on everything that happens to be in a buffer.
+This way you can have Flymake enabled globally and still not run
+C<perl -c> on everything that happens to be in a buffer.
 
 
 
@@ -1088,7 +1133,7 @@ is customized to a true value.
 =head3 Code Coverage Visualization Introduction
 
 
-=for html <p>[ <a href="http://search.cpan.org/src/JOHANL/Devel-PerlySense-0.0183/doc/code_coverage.html">Screenshot</a> ]<p>
+=for html <p>[ <a href="http://search.cpan.org/src/JOHANL/Devel-PerlySense-0.0217/doc/code_coverage.html">Screenshot</a> ]<p>
 
 
 If you have a test suite, you might like this. You should have tests.
@@ -1120,6 +1165,16 @@ ordinarily the case).
 Note: Running the test suite with Devel::Cover can be very, very
 slow. A nightly build is usually a good idea.
 
+You can also collect / undate coverage information for indivual test
+files with C<C-o r c>. This is the easiest way to just try it out.
+
+You might want to add the following to be ignored by your VCS
+(e.g. .gitignore):
+
+    /cover_db/*
+    /covered/*
+
+
 
 =head3 Using Coverage Visualization
 
@@ -1169,8 +1224,51 @@ I<that particular subroutine>.
 
 =head2 Editing Code
 
+Editing code includes both smaller editing tasks and refactorings to
+restucture the code.
 
-=head3 Edit Add 'use Module' Statement
+
+=head3 Edit - Copy Package Name
+
+C<C-o e c p> -- Copy the current package statement name to the
+clipboard (kill-ring) and display it in the echo area. If there is no
+package statement, try to get the package name from the file name.
+
+
+
+=head3 Edit - Copy Package Name From File Name
+
+C<C-o e c P> -- Assuming the file is a Perl module in a lib directory,
+copy the corrsponding package name to the clipboard (kill-ring) and
+display it in the echo area.
+
+Useful when you've just created an empty new Perl module .pm file.
+
+
+
+=head3 Edit - Copy Sub Name
+
+C<C-o e c s> -- Copy the current sub name to the clipboard (kill-ring)
+and display it in the echo area.
+
+
+
+=head3 Edit - Copy Method Name
+
+C<C-o e c m> -- Copy the current method name to the clipboard
+(kill-ring) and display it in the echo area. Method name in this case
+means "package->sub".
+
+
+
+=head3 Edit - Copy File Name
+
+C<C-o e c f> -- Copy the current file name to the clipboard
+(kill-ring) and display it in the echo area.
+
+
+
+=head3 Edit - Add 'use Module' Statement
 
 C<C-o e a u> -- Set mark and add a 'use My::Module;' statement to the
 end of the 'use Module' section at the top of the file.
@@ -1188,7 +1286,7 @@ you where doing.
 
 
 
-=head3 Edit Move 'use Module' Statement
+=head3 Edit - Move 'use Module' Statement
 
 C<C-o e m u> -- If point is on a line with a single 'use Module'
 statement, set mark and move that statement to the end of the 'use
@@ -1202,11 +1300,172 @@ that it got moved to a good place and hit C-u C-SPC to return to where
 you were, and continue doing what you where doing.
 
 
+=head3 Edit/Refactor - Extract Variable
+
+C<C-o e e v> -- Do the refactoring Extract Variable of the active region.
+
+For example, in this piece of code:
+
+    my $syntax = $self->perlysense->config->{external}->{editor}->{emacs}->{flymake}->{syntax};
+    my $critic = $self->perlysense->config->{external}->{editor}->{emacs}->{flymake}->{critic};
+
+Select a piece of code (on either of the lines) that is duplicated a
+lot and hit C<C-o e e v>. In this case this seems to be the common
+part:
+
+    $self->perlysense->config->{external}->{editor}->{emacs}->{flymake}
+
+You will be asked for a variable name to put this in. The default is
+the last word in the selected code ($flymake).
+
+All occurrences of the selection will now be replaced with $flymake,
+and the new variable $flymake will be declared just before the
+earliest usage.
+
+    my $flymake = $self->perlysense->config->{external}->{editor}->{emacs}->{flymake};
+    my $syntax = $flymake->{syntax};
+    my $critic = $flymake->{critic};
+
+Before the edit, the C<mark> was pushed at the location where you
+started, so you can hit C<C-u C-SPC> to jump back.
+
+After the edit, the point is left at the new variable declaration so
+you can ensure that it is in a reasonable location. It's not unusual
+to need to move it to an outer scope in order for all the usages to be
+covered by the declaration.
+
+Now you need to ensure this edit makes sense. Both replacements and
+the declaration are highlighted, so it's easy to see what was
+changed.
+
+Once you've eye-balled the edits, hit C<C-o e h> to remove the
+Highlights.
+
+Note that the replacement is syntax unaware, so you'll have to ensure
+it's syntactically correct yourself (althugh most of the time it works
+just fine).
+
+In this particular example, had there been no arrows between the hash
+keys, the final code would have looked like this:
+
+    my $flymake = $self->perlysense->config->{external}{editor}{emacs}{flymake};
+    my $syntax = $flymake{syntax};
+    my $critic = $flymake{critic};
+
+and that clearly isn't equivalent Perl code, the flymake hashref
+having been converted to a hash. This is probably the most common
+failure mode though, and shouldn't happen that often. Now you know.
+
+By default, only the current subroutine is changed. Invoke with the
+prefix arg to change the entire buffer: C<C-u C-o e e v>.
+
+Cool usages for Extract Variable:
+
+=over 4
+
+=item * Remove duplicated code (duh), beause duplication is just shoddy.
+
+=item * Rename variable - Extract Varable, then just delete the declaration.
+
+=item * C<print "So, you want to make a $object-E<gt>method_call inside a string\n";>
+
+But that doesn't work obviously. So you mark C<$object-E<gt>method_call>
+and extract it, and end up with this:
+
+    my $method_call = $object->method_call;
+    print "So, you want to make a $method_call inside a string\n";
+
+Nice!
+
+=back
+
+
+
+=head3 Edit -- Find Callers
+
+C<C-o e f c> -- Find callers of a method in the current project, and
+insert the package->sub as a call tree in a comment.
+
+This is for understanding where in the code base method calls
+originate.
+
+If point is in a comment on something that looks like a method call,
+look for that method. This can be in source code, or in a comment with
+callers. Insert the comment with callers above the current line.
+
+Otherwise, look for callers to the current sub. Insert the comment
+with callers above the sub declaration.
+
+Example: Point is in the sub C<price>:
+
+    package MyApp::Book;
+
+    sub price {
+    |
+
+Hit C<C-o e f c> and PerlySense will insert the three places where the
+price method is called:
+
+    package MyApp::Book;
+
+    #     MyApp::Book->discount_price
+    #     MyApp::User->total_book_cost
+    #    |MyApp::Author->daily_total_income
+    # MyApp::Book->price
+    sub price {
+
+Let's assume the method call chain for total_book_cost is interesting,
+so put the cursor on that line and again hit C<C-o e f c>. The callers
+for that method is now inserted on the line above.
+
+    package MyApp::Book;
+
+    #     MyApp::Book->discount_price
+    #         MyApp::Controller::User->user_details
+    #        |MyApp::User->total_cost
+    #     MyApp::User->total_book_cost
+    #     MyApp::Author->daily_total_income
+    # MyApp::Book->price
+    sub price {
+
+You can go on like this and add more callers to investigate the code
+structure.
+
+The cursor is placed conveniently to make it easy to add subsequent
+callers to the call tree.
+
+If the same caller is already present in the comment, it is marked
+with a * to indicate that there's no point following them.
+
+Caveat: The method of identifying callers works by method names alone,
+so there might be false positives, or uninteresting callers added to
+the list. Delete those lines to avoid clutter.
+
+
+
+=head3 Edit -- Visualize Callers
+
+C<C-o e v c> -- Visualize callers in a call tree comment (collected
+using Find Callers above) by drawing it using GraphViz.
+
+Put the cursor in a comment with the call tree and hit C<C-o e v
+c>. PerlySense will create a temporary .dot file and let GraphViz
+render it into a nice .png image, which will be opened.
+
+If you're running a graphical Emacs it might even look pretty.
+
+This requires GraphViz' C<dot> binary to be installed:
+
+    sudo apt-get install graphviz  # Debian / Ubuntu
+    sudo yum install graphviz      # Redhat / CentOS
+
+on OSX, try brew something.
+
 
 =head3 Assist With -- Regex
 
 
-=for html <p>[ <a href="http://search.cpan.org/src/JOHANL/Devel-PerlySense-0.0183/doc/regex_tool.html">Screenshot</a> ]<p>
+=for html <p>[ <a href="http://search.cpan.org/src/JOHANL/Devel-PerlySense-0.0217/doc/regex_tool.html">Screenshot</a> ]<p>
 
 
 Hit C<C-o a r> to bring up the Regex Tool which will let you compose a
@@ -1493,15 +1752,10 @@ Bring up an overview of things.
 Reformat source.
 
 
-=item * e -- Edit
-
-Perform smaller convenience editing task.
-
-
-=item * E -- rEfactor
-
-Perform restructuring edits that don't impact functionality/behaviour.
+=item * e -- Edit & Refactor
 
+Perform smaller convenience editing task, as well as refactorings --
+restructuring edits that don't impact functionality/behaviour.
 
 =item * A -- Assist
 
@@ -1590,9 +1844,11 @@ L<http://vimdoc.sourceforge.net/htmldoc/eval.html#Dictionaries> - Vim native dat
 
 
 
+=encoding utf8
+
 =head1 AUTHOR
 
-Johan Lindstrm, C<< <johanl buzzwordninja.com> >>
+Johan Lindstrom, C<< <johanl buzzwordninja.com> >>
 
 =head1 CONTRIBUTIONS, BUGS, AND CAVEATS
 
@@ -1644,7 +1900,7 @@ Jaeyoun Chung for dropdown-list L<http://www.emacswiki.org/cgi-bin/wiki/dropdown
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2007 Johan Lindstrm, All Rights Reserved.
+Copyright 2007 Johan Lindstrom, All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -1657,12 +1913,10 @@ under the same terms as Perl itself.
 
 use strict;
 use warnings;
+use utf8;
 
 package Devel::PerlySense;
-BEGIN {
-  $Devel::PerlySense::VERSION = '0.0205'; # TRIAL
-}
-
+$Devel::PerlySense::VERSION = '0.0222'; # TRIAL
 
 
 use Spiffy -Base;
@@ -1672,6 +1926,7 @@ use File::Basename;
 use File::Path;
 use File::Find::Rule;
 use Path::Class qw/dir file/;
+use Path::Tiny;
 use Pod::Text;
 use IO::String;
 use Cache::Cache;
@@ -1691,6 +1946,8 @@ use Devel::PerlySense::Class;
 use Devel::PerlySense::Document;
 use Devel::PerlySense::Document::Location;
 use Devel::PerlySense::BookmarkConfig;
+use Devel::PerlySense::CallTree;
+use Devel::PerlySense::CallTree::Graph;
 
 
 
@@ -1934,6 +2191,9 @@ Currently supported:
   $self->method, look in current file and base classes. If no sub can
   be found, look for POD.
 
+  shift->method for subs that don't have a $self. Same as
+  $self->method.
+
   $object->method, look in current file and used modules. If no sub
   can be found, look for POD.
 
@@ -2489,6 +2749,95 @@ sub isFileInProject {
 
 
 
+=head2 raCallSiteForMethod(method => $nameMethod, dirOrigin => $dirOrigin)
+
+Find callers of $nameMethod in $dirOrigin.
+
+Return array ref of call sites.
+
+=cut
+sub raCallSiteForMethod {
+    my ($nameMethod, $dirOrigin) = Devel::PerlySense::Util::aNamedArg(["nameMethod", "dirOrigin"], @_);
+
+    $self->setFindProject(dir => $dirOrigin);
+
+    my @aMatch;
+    my %hSeen;
+    for my $file ( $self->oProject->aFileSourceCode ) {
+        my $source = slurp($file);
+
+        my $row = 0;
+        my $oDocument;
+        for my $line (split("\n", $source)) {
+            $row++;
+            $line =~ m/ -> \s* $nameMethod \b /x or next;
+            $line =~ m/ ^ \s* \# /x and next; # No comments
+            $oDocument ||= $self->oDocumentParseFile($file) or last;
+
+            my $oLocationSub = $oDocument->oLocationSubAt(
+                row => $row,
+                col => 1,
+            ) or next;
+
+            my $rhPropertySub = $oLocationSub->rhProperty;
+            my $namePackage = $rhPropertySub->{namePackage};
+            my $nameSub = $rhPropertySub->{nameSub};
+
+            $hSeen{ "$namePackage->$nameSub" }++ and next;
+
+            push(
+                @aMatch,
+                {
+                    file    => $file,
+                    package => $namePackage,
+                    method  => $nameSub,
+                },
+            );
+        }
+    }
+
+    ###JPL: make any file names relative to project
+
+    return \@aMatch;
+}
+
+
+
+
+
+=head2 rhFileCallerVisualized(source => $source)
+
+Extract call tree from $source and render it into a .dot and .png
+file.
+
+Return hash ref with (keys: "dot", "image"; values: file names).
+
+Die if there is no "dot" binary to run.
+
+=cut
+sub rhFileCallerVisualized {
+    my ($source) = Devel::PerlySense::Util::aNamedArg(["source"], @_);
+
+    # TODO: extract
+    my $dirTemp = path("~/.PerlySense/temp");
+    my $dirTempCallTree = path($dirTemp, "call_tree");
+
+    my $treeCallers = Devel::PerlySense::CallTree->new(source => $source);
+    my $graph = Devel::PerlySense::CallTree::Graph->new({
+        call_tree  => $treeCallers,
+        output_dir => $dirTempCallTree,
+    });
+    $graph->create_graph();
+
+    return {
+        dot   => $graph->dot_file . "",
+        image => $graph->output_file . "",
+    };
+}
+
+
+
+
 =head1 IMPLEMENTATION METHODS
 
 =head2 fileFindLookingAround($fileModuleBase, $dirOrigin, $nameModule?)
