@@ -30,12 +30,10 @@ no good way to expire obsolete files.
 
 use strict;
 use warnings;
+use utf8;
 
 package Devel::PerlySense::Document;
-BEGIN {
-  $Devel::PerlySense::Document::VERSION = '0.0205'; # TRIAL
-}
-
+$Devel::PerlySense::Document::VERSION = '0.0222'; # TRIAL
 
 
 
@@ -430,6 +428,8 @@ sub methodCallAt {
 
 Return the name of the $self->method at $row, $col in this document.
 
+Also matches shift->method, if there is no $self in this sub at all.
+
 If no method call is found, maybe warn and return undef.
 
 Die on errors.
@@ -440,14 +440,40 @@ sub selfMethodCallAt {
 
     my ($object, $method) = $self->methodCallAt(row => $row, col => $col);
     $method or return(undef);
-    $object and $object eq '$self' or return(undef);    #We only know about self so far
+    $object or return(undef);
+    $object eq '$self' and return($method);
 
-    return($method);
+    # If the object is "shift" and there is no mention of a $self in
+    # the sub, assume it's $self being shifted off @_
+    if($object eq "shift") {
+        $self->isThereSelfInSubAt(row => $row, col => $col) and return undef;
+        return($method);
+    }
+
+    return(undef);
 }
 
+=head2 isThereSelfInSubAt(row => $row, col => $col) : Bool
 
+Whether there is a mention of $self in the sub surrounding $row, $col.
 
+=cut
+sub isThereSelfInSubAt {
+    my ($row, $col) = Devel::PerlySense::Util::aNamedArg(["row", "col"], @_);
 
+    my $oLocationSubAt = $self->oLocationSubAt(row => $row, col => $col)
+        or return(0);
+
+    my $source = $oLocationSubAt->rhProperty->{source} or return(0);
+
+    if ( $source =~ / \$self \b /smx ) {
+        # There is a $self somewhere in this sub (could be false
+        # positive in a comment or string), so shift isn't $self
+        return(1);
+    }
+
+    return(0);
+}
 
 =head2 moduleMethodCallAt(row => $row, row => $col)
 
@@ -464,7 +490,7 @@ sub moduleMethodCallAt {
 
     my ($module, $method) = $self->methodCallAt(row => $row, col => $col);
     $module && $method or return(undef);
-    $module =~ /[^\w:]/ and return(undef);   #only allow bareword modules
+    $module =~ /[^\w:]/ and return(undef); #only allow bareword modules
 
     wantarray() and return($module, $method);
     return("$module->$method");
@@ -517,7 +543,7 @@ Die if $file doesn't exist, or on other errors.
 =cut
 sub rhRegexExample {
     my ($row, $col) = Devel::PerlySense::Util::aNamedArg(["row", "col"], @_);
-    
+
     return { regex => "", example => "" };
 }
 
@@ -576,11 +602,11 @@ sub oLocationSubAt {
         if(           $row >= $oLocation->row
                    && $row <= $oLocation->rhProperty->{oLocationEnd}->row
                ) {
-            debug("Sub found at ($row/$col): (" . Dumper($oLocation) . ")");
+            debug("Sub " . $oLocation->rhProperty->{namePackage} . "->" . $oLocation->rhProperty->{nameSub} . " found at (" . $oLocation->file . ":$row)");
             return($oLocation->clone);
         }
     }
-    
+
     return(undef);
 }
 
@@ -790,7 +816,7 @@ sub determineLikelyApi {
         $self->rhPackageApiLikely($rhPackageApi);
     } else {
         $self->determineLikelyApi0(nameModule => $nameModule);
-       $self->cacheSet($keyCache, $self->file, $self->rhPackageApiLikely);
+        $self->cacheSet($keyCache, $self->file, $self->rhPackageApiLikely);
    }
 
     return(1);
@@ -983,7 +1009,10 @@ my $matchReplace = {
     q/sub\s+\w+\s*:\s*\w+[^{]+{/ => q/SPECIAL/,
     q/^=(?:head|item|for|pod)/ => q/SPECIAL/,
 };
-my $rexMatch = join("|", keys %$matchReplace );
+my $rexMatch = join(
+    "|",
+    map { s/\{/\\{/; $_; } keys %$matchReplace,
+);
 sub _stringReplace {
     my ($match) = @_;
 
@@ -1171,9 +1200,11 @@ sub cacheGet {
 
 __END__
 
+=encoding utf8
+
 =head1 AUTHOR
 
-Johan Lindstrm, C<< <johanl[T]DarSerMan.com> >>
+Johan Lindstrom, C<< <johanl@cpan.org> >>
 
 =head1 BUGS
 
@@ -1187,7 +1218,7 @@ your bug as I make changes.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2005 Johan Lindstrm, All Rights Reserved.
+Copyright 2005 Johan Lindstrom, All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
