@@ -16,12 +16,10 @@ _may_ support).
 
 use strict;
 use warnings;
+use utf8;
 
 package Devel::PerlySense::Document::Api;
-BEGIN {
-  $Devel::PerlySense::Document::Api::VERSION = '0.0205'; # TRIAL
-}
-
+$Devel::PerlySense::Document::Api::VERSION = '0.0222'; # TRIAL
 
 
 
@@ -377,9 +375,11 @@ sub percentConsistsOf {
 
 __END__
 
+=encoding utf8
+
 =head1 AUTHOR
 
-Johan Lindstrm, C<< <johanl[T]DarSerMan.com> >>
+Johan Lindstrom, C<< <johanl@cpan.org> >>
 
 =head1 BUGS
 
@@ -393,7 +393,7 @@ your bug as I make changes.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2005 Johan Lindstrm, All Rights Reserved.
+Copyright 2005 Johan Lindstrom, All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
