@@ -21,12 +21,10 @@ A Project has configuration settings.
 
 use strict;
 use warnings;
+use utf8;
 
 package Devel::PerlySense::Project;
-BEGIN {
-  $Devel::PerlySense::Project::VERSION = '0.0205'; # TRIAL
-}
-
+$Devel::PerlySense::Project::VERSION = '0.0222'; # TRIAL
 
 
 use Spiffy -Base;
@@ -36,6 +34,7 @@ use File::Basename;
 use Path::Class;
 use File::Slurp;
 use File::chdir;
+use File::Find::Rule;
 use Perl::Critic 1.105;
 use List::MoreUtils qw/ uniq /;
 
@@ -533,6 +532,7 @@ Return 1 or die on errors (but that will look like the result of a
 failed flymake run).
 
 =cut
+
 sub flymakeFile {
     my ($file) = Devel::PerlySense::Util::aNamedArg(["file"], @_);
 
@@ -655,6 +655,8 @@ sub raFileTestOther {
     $file =~ m|\.pm$| and $method = "test_files_covering";     # If a module, it's source
     $file =~ m|/t/|   and $method = "source_files_covered_by"; # If it's in /t/, it's a test
     $file =~ m|\.t$|  and $method = "source_files_covered_by"; # If it's a .t file, it's a test
+    ### TODO: config with list of lib dirs and list of test dirs
+    debug("raFileTestOther with method ($method)");
 
     my $fileRelative = file($file)->relative( $self->dirProject );
     my $raFileTestOther = [ $db->$method($fileRelative, $sub) ];
@@ -708,6 +710,23 @@ sub raFileProjectOther {
 
 
 
+=head2 aFileSourceCode()
+
+Return array with file names with perl source code in the project.
+
+# TODO: also look inside files without extension and check for perlyness
+
+=cut
+sub aFileSourceCode {
+    return File::Find::Rule
+        ->name("*.pm", "*.pl")
+        ->in( $self->aDirIncAbsolute );
+}
+
+
+
+
+
 
 1;
 
@@ -717,9 +736,11 @@ sub raFileProjectOther {
 
 __END__
 
+=encoding utf8
+
 =head1 AUTHOR
 
-Johan Lindstrm, C<< <johanl[T]DarSerMan.com> >>
+Johan Lindstrom, C<< <johanl@cpan.org> >>
 
 =head1 BUGS
 
@@ -733,7 +754,7 @@ your bug as I make changes.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2005 Johan Lindstrm, All Rights Reserved.
+Copyright 2005 Johan Lindstrom, All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
