@@ -18,9 +18,7 @@ A Project::Unknown indicates the lack of a project being defined.
 
 
 package Devel::PerlySense::Project::Unknown;
-BEGIN {
-  $Devel::PerlySense::Project::Unknown::VERSION = '0.0205'; # TRIAL
-}
+$Devel::PerlySense::Project::Unknown::VERSION = '0.0222'; # TRIAL
 use base "Devel::PerlySense::Project";
 
 
@@ -32,6 +30,7 @@ use strict;
 use warnings;
 use Carp;
 use Data::Dumper;
+use utf8;
 
 
 
@@ -62,9 +61,11 @@ sub dirProject {
 
 __END__
 
+=encoding utf8
+
 =head1 AUTHOR
 
-Johan Lindstrm, C<< <johanl[T]DarSerMan.com> >>
+Johan Lindstrom, C<< <johanl@cpan.org> >>
 
 =head1 BUGS
 
@@ -78,7 +79,7 @@ your bug as I make changes.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2005 Johan Lindstrm, All Rights Reserved.
+Copyright 2005 Johan Lindstrom, All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
